#!/usr/local/bin/perl
# bwgraph.cgi
# Show current bandwidth usage graphs

require './virtual-server-lib.pl';
&ReadParse();
use POSIX;

# Work out which domains to show
if ($in{'dom'}) {
	@doms = ( &get_domain($in{'dom'}) );
	$subh = &domain_in($doms[0]);
	}
else {
	@doms = grep { !$_->{'parent'} } &list_domains();
	}
foreach $d (@doms) {
	&can_edit_domain($d) || &error($text{'newbw_ecannot'});
	}
&ui_print_header($subh, $text{'bwgraph_title'}, "");

# Show current usage and limit for all virtual servers
$max = 0;
foreach $d (@doms) {
	$max = $d->{'bw_usage'} if ($d->{'bw_usage'} > $max);
	$max = $d->{'bw_limit'} if ($d->{'bw_limit'} > $max);
	}
if ($max) {
	# Show links for mode
	print "<b>$text{'bwgraph_mode'}</b>\n";
	foreach $m (0, 1, 2) {
		print "&nbsp;|&nbsp;\n" if ($m);
		$t = $text{'bwgraph_mode_'.$m};
		if ($m == $in{'mode'}) {
			print $t;
			}
		else {
			print "<a href='bwgraph.cgi?mode=$m&dom=$in{'dom'}'>$t</a>\n";
			}
		}
	print "<p>\n";

	# Show table
	$width = 500;
	print "<table>\n";
	if ($in{'mode'} == 0 || $in{'mode'} == 1) {
		# By domain
		print "<tr> <td><b>$text{'newbw_dom'}</b></td>\n";
		print "<td><b>",&text('edit_bwpast_'.$config{'bw_past'},
				      $text{'newbw_graph'}, $config{'bw_period'}),
		      "</b></td>\n";
		print "<td><b>$text{'newbw_glimit'}</b></td>\n";
		print "<td><b>$text{'newbw_gusage'}</b></td>\n";
		print "</tr>\n";
		foreach $d (sort { $b->{'bw_usage'} <=> $a->{'bw_usage'} }
			    grep { !$_->{'parent'} }
			    @doms) {
			$usage = $in{'mode'} == 1 ? $d->{'bw_usage_only'}
						  : $d->{'bw_usage'};
			if ($in{'dom'}) {
				print "<tr> <td>$d->{'dom'}</td> <td nowrap>\n";
				}
			else {
				print "<tr> <td><a href='bwgraph.cgi?dom=$d->{'id'}'>",
				      "$d->{'dom'}</td> <td nowrap>\n";
				}

			# Show limit, or grey box if unlimited
			if ($d->{'bw_limit'}) {
				printf "<img src=images/red.gif width=%s height=10>\n",
					int($width*$d->{'bw_limit'}/$max)+1;
				}
			else {
				printf "<img src=images/grey.gif width=%s height=10>\n",
					$width;
				}

			# Show usage by feature
			print "<br>";
			&usage_colours($d, $in{'mode'});

			print "</td>\n";
			print "<td>\n";
			if ($d->{'bw_limit'}) {
				print &nice_size($d->{'bw_limit'});
				}
			else {
				print $text{'newbw_unlim'};
				}
			print "</td>\n";
			print "<td>",&nice_size($usage),"</td>\n";
			print "</tr>\n";

			next if ($in{'mode'} != 1);

			# Show sub-servers
			$space = $d->{'bw_usage_only'};
			foreach $sd (grep { !$_->{'alias'} }
					  &get_domain_by("parent", $d->{'id'})) {
				print "<tr> <td>&nbsp;&nbsp;&nbsp;",
				      "$sd->{'dom'}</td> <td>\n";
				printf "<img src=images/white.gif width=%s height=10>",
					int($width*$space/$max);
				&usage_colours($sd, 1);
				print "</td>\n";
				print "<td></td>\n";
				print "<td>",&nice_size($sd->{'bw_usage_only'}),"</td>\n";
				print "</tr>\n";
				$space += $sd->{'bw_usage_only'};
				}
			}
		}
	elsif ($in{'mode'} == 2) {
		# By date, for current billing period
		print "<tr> <td><b>$text{'newbw_date'}</b></td>\n";
		print "<td><b>$text{'newbw_dusage'}</b></td>\n";
		print "<td><b>$text{'newbw_gusage'}</b></td>\n";
		print "</tr>\n";

		# Get bandwidth for relevant domains and sub-domains
		foreach $d (@doms) {
			foreach $dd ($d, &get_domain_by("parent", $d->{'id'})) {
				local $bwinfo = &get_bandwidth($dd);
				push(@bands, $bwinfo);
				}
			}

		# Work out the max day
		$now = time();
		$day = int($now / (24*60*60));
		$start_day = &bandwidth_period_start();
		$max = 0;
		for($i=$day; $i>=$start_day; $i--) {
			$usage = 0;
			foreach $f (@features) {
				foreach $band (@bands) {
					$usage += $band->{$f.'_'.$i};
					}
				}
			$max = $usage if ($usage > $max);
			}

		# Show the day table
		$max ||= 1;
		for($i=$day; $i>=$start_day; $i--) {
			print "<tr>\n";
			print "<td>",strftime("%d/%m/%Y", localtime($i*24*60*60)),"</td>\n";
			print "<td>";
			local $usage = 0;
			foreach $f (@features) {
				foreach $band (@bands) {
					$usage += $band->{$f.'_'.$i};
					if ($usage) {
						printf "<img src=images/usage-$f.gif width=%s height=10>",
							int($width*$band->{$f.'_'.$i}/$max)+($f eq "web" ? 1 : 0);
						$donecolour{$f} += $band->{$f.'_'.$i};
						}
					}
				}
			print "</td>";
			print "<td>",&nice_size($usage),"</td>\n";
			print "</tr>\n";
			}
		}
	print "</table>\n";

	# Show colour keys
	print "<br>\n";
	foreach $f (@features) {
		if ($donecolour{$f}) {
			print "<img src=images/usage-$f.gif width=10 height=10>\n";
			local $label = $text{'bandwidth_'.$f} ||
				       $text{'feature_'.$f};
			print $label," (",
			      &nice_size($donecolour{$f}),")\n";
			}
		}
	print "<br>\n";
	}
else {
	print "<b>$text{'bwgraph_none'}</b><p>\n";
	}

if ($in{'dom'} && &master_admin()) {
	&ui_print_footer("bwgraph.cgi", $text{'bwgraph_return'},
			 "", $text{'index_return'});
	}
else {
	&ui_print_footer("", $text{'index_return'});
	}

# usage_colours(&domain, mode)
sub usage_colours
{
local ($d, $mode) = @_;
local $f;
foreach $f (@features) {
	local $fusage = $mode == 1 ? $d->{'bw_usage_only_'.$f}
			     	   : $d->{'bw_usage_'.$f};
	if ($fusage) {
		printf "<img src=images/usage-$f.gif width=%s height=10>",
			int($width*$fusage/$max)+($f eq "web" ? 1 : 0);
		$donecolour{$f} += $fusage;
		}
	}
}

